/********************************************************************************
    Copyright (C) 1999  Dirk Farin

    This program is distributed under GNU Public License (GPL) as
    outlined in the COPYING file that comes with the source distribution.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ********************************************************************************/

#include "output/mgavid.hh"

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/mman.h>
#include <stdio.h>


VideoOutput_MGA::VideoOutput_MGA()
  : first(true), scaled(false), width(0), height(0)
{
  fileh = open("/dev/mga_vid",O_RDWR);
}


VideoOutput_MGA::~VideoOutput_MGA()
{
  if (fileh>0) close(fileh);
}


void VideoOutput_MGA::Show_const(const Image_YUV<Pixel>& img)
{
#if 0
  static cnt=0;
  cnt++;
  if (!(cnt%100)) cout << cnt << endl;
  return;
#endif

  assert(fileh>0);

  if (first)
    {
      first=false;
      ImageParam_YUV param;
      img.GetParam(param);

      config.src_width   = param.width;
      config.src_height  = param.height;
      if (scaled)
	{
	  config.dest_width  = width;
	  config.dest_height = height;
	}
      else
	{
	  config.dest_width  = param.width;
	  config.dest_height = param.height;
	}

      config.x_org = 0;
      config.y_org = 0;
      config.colkey_on = 0;

      if (ioctl(fileh,MGA_VID_CONFIG,&config))
	{
	  perror("error in MGA_VID config.\n");
	}

      ioctl(fileh,MGA_VID_ON,0);
      base = (uint_8*)mmap(0,256*4096,PROT_WRITE,MAP_SHARED,fileh,0);
    }

  const Pixel*const* yp = img.AskFrameY_const();
  const Pixel*const* up = img.AskFrameU_const();
  const Pixel*const* vp = img.AskFrameV_const();
  if (config.card_type == MGA_G400)
    {
      int bes_pitch = (config.src_width+31) & ~31;
      uint_8* dest = base;

      for (int h=0;h<config.src_height;h++)
	{
	  memcpy(dest,yp[h],config.src_width);
	  dest += bes_pitch;
	}

      for (int h=0;h<config.src_height/2;h++)
	{
	  memcpy(dest,up[h],config.src_width/2);
	  dest += bes_pitch/2;
	}
      for (int h=0;h<config.src_height/2;h++)
	{
	  memcpy(dest,vp[h],config.src_width/2);
	  dest += bes_pitch/2;
	}
    }
  else
    {
      throw "NO G400\n";
    }
}

